package com.yizhi.documents.application.controller;


import com.yizhi.documents.application.domain.DocumentClassify;
import com.yizhi.documents.application.service.ClassifyService;
import com.yizhi.documents.application.service.DocumentService;
import com.yizhi.documents.application.vo.documents.DocumentClassifyTreeVo;
import com.yizhi.documents.application.vo.documents.DocumentClassifyVo;
import com.yizhi.documents.application.vo.documents.ParamsVo;
import com.yizhi.documents.application.vo.documents.SecondClassifyVo;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author hutao
 * @since 2019-06-11
 */
@RestController
@RequestMapping("/classify")
public class ClassifyComtroller {

    @Resource
    private ClassifyService classifyService;
    @Resource
    private DocumentService documentService;


    @PostMapping("/save")
    public ParamsVo save(@RequestBody DocumentClassify classify) {

        ParamsVo paramsVo = new ParamsVo();

        if (null != classify.getParentId()) {
            DocumentClassify classifyTem = classifyService.selectById(classify.getParentId());
            if (null != classifyTem) {
                if (classifyTem.getLayer() == 3) {
                    paramsVo.setErrorMsg("分类最多为三级！");
                    paramsVo.setSuccess(false);
                    return paramsVo;
                }
            }
        }
        if (classifyService.save(classify)) {
            paramsVo.setSuccess(true);
            paramsVo.setErrorMsg(null);
        }
        return paramsVo;
    }

    @PostMapping("/update")
    public Boolean update(@RequestBody DocumentClassifyVo classifyVo) {

        if (classifyService.update(classifyVo)) {
            return true;
        }
        return false;
    }

    @GetMapping("/delete")
    public ParamsVo delete(@RequestParam("id") Long id) {


        ParamsVo paramsVo = new ParamsVo();
        if (null != id) {
            DocumentClassify classify = classifyService.selectById(id);
            if (classify == null) {
                paramsVo.setErrorMsg("数据库不存在该id！");
                paramsVo.setSuccess(false);
            } else {
                //判断是否关联了资料
                Integer i = documentService.queryRelationCount(id);
                //判断是否有子分类
                Boolean hasChildrenClassify = classifyService.haveChildrenClassify(id);
                if (hasChildrenClassify || i > 0) {
                    paramsVo.setSuccess(false);
                    paramsVo.setErrorMsg("该分类下有子分类或者已关联资料，无法删除！");
                } else {
                    if (classifyService.delete(classify)) {
                        paramsVo.setSuccess(true);
                        paramsVo.setErrorMsg(null);
                    } else {
                        paramsVo.setErrorMsg("删除失败！");
                        paramsVo.setSuccess(false);
                    }
                }
            }
        }
        return paramsVo;
    }

    /**
     * 查询 树形结构的分类
     *
     * @param showThirdClassfy 是否显示 第三级分类
     * @return
     */
    @GetMapping("/query")
    public List<DocumentClassifyTreeVo> queryClassfy(@RequestParam("showThirdClassfy") Boolean showThirdClassfy) {
        List<DocumentClassifyTreeVo> classifyTreeVos = classifyService.queryClassifyList();
        ;
        if (!showThirdClassfy) {
            classifyTreeVos.forEach(a -> {
                List<SecondClassifyVo> children = a.getChildren();
                children.forEach(b -> {
                    b.setChildren(new ArrayList<>());
                });
            });
        }

        return classifyTreeVos;
    }

    @GetMapping("/queryById")
    public DocumentClassifyVo queryById(@RequestParam("id") Long id) {

        DocumentClassify classify = this.classifyService.selectById(id);
        if (classify != null) {
            DocumentClassifyVo classifyVo = new DocumentClassifyVo();

            classifyVo.setId(classify.getId());
            classifyVo.setName(classify.getName());
            classifyVo.setDescription(classify.getDescription());
            if (classify.getLayer() != 1) {
                DocumentClassify parentClassify = this.classifyService.selectById(classify.getParentId());
                if (parentClassify != null) {
                    classifyVo.setParentName(parentClassify.getName());
                }
            }
            return classifyVo;
        }
        return null;
    }

}