package com.yizhi.documents.application.controller;


import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.documents.application.domain.Document;
import com.yizhi.documents.application.download.DocumentListExcel;
import com.yizhi.documents.application.mapper.DocumentMapper;
import com.yizhi.documents.application.service.DocumentRelationService;
import com.yizhi.documents.application.service.DocumentService;
import com.yizhi.documents.application.util.WorkUtil;
import com.yizhi.comment.application.feign.PdfPagesClient;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.documents.application.vo.documents.*;
import com.yizhi.util.application.constant.TpActivityType;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.util.CollectionUtils;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * <p>
 * 前端控制器
 * </p>
 *
 * @author hutao
 * @since 2019-06-11
 */
@RestController
@RequestMapping("/document")
public class DocumentComtroller {


    @Autowired
    private DocumentService documentService;

    @Autowired
    private DocumentRelationService documentRelationService;

    @Autowired
    private DocumentListExcel documentListExcel;
    @Autowired
    private WorkUtil workUtil;
    @Autowired
    private DocumentMapper documentMapper;

    @Autowired
    private PdfPagesClient pdfPagesClient;
    private Logger LOGGER =  LoggerFactory.getLogger(DocumentComtroller.class);


    @PostMapping("/upload")
    public Boolean upload(@RequestBody UploadVo uploadVo) {
        return documentService.upload(uploadVo);
    }

    @GetMapping("/delete")
    public ParamsVo delete(@RequestParam("documentId") Long documentId,
                           @RequestParam("needDelete") Boolean needDelete) {
        ParamsVo paramsVo = new ParamsVo();
        Document document = documentService.selectById(documentId);
        if (document == null) {
            paramsVo.setErrorMsg("数据库不存在该id！");
            paramsVo.setSuccess(false);
            return paramsVo;
        } else {
            document.setDeleted(1);

            if (documentRelationService.hasRelation(documentId)) {
                paramsVo.setErrorMsg("该资料已被课程和培训项目引用，若继续删除将同时删除对应引用关系");
                paramsVo.setSuccess(false);
                //是否强制删除  则删除资料及其关联关系
                if (needDelete) {
                    if (documentRelationService.delete(null, documentId) && documentService.updateById(document)) {
                        paramsVo.setSuccess(true);
                        paramsVo.setErrorMsg(null);
                        return paramsVo;
                    }
                }
            } else {
                if (needDelete) {
                    if (documentService.updateById(document)) {
                        paramsVo.setSuccess(true);
                        paramsVo.setErrorMsg(null);
                    }
                } else {
                    paramsVo.setErrorMsg("确定删除该资料，删除后不可恢复。");
                    paramsVo.setSuccess(false);
                }
            }
        }
        return paramsVo;
    }

    @PostMapping("/query/onCondition")
    public Page<DocumentVo> query(@RequestBody DocumentParamVo documentParamVo) {

        if (null != documentParamVo) {
            RequestContext context = ContextHolder.get();
            documentParamVo.setContext(context);
            return documentService.queryOnCondition(documentParamVo);

        }
        return null;
    }

    @PostMapping("/update")
    public ParamsVo update(@RequestBody UploadVo uploadVo) {

        return documentService.update(uploadVo);
    }

    /**
     * @param
     * @return
     */
    @GetMapping("/query/byId")
    public DocumentVo queryById(@RequestParam Long id) {
    	DocumentVo vo = new DocumentVo();
    	List<PdfPage> pages = new ArrayList<PdfPage>();
        if (null != id) {
            Document document = documentService.selectById(id);
            if (document == null) {
                return null;
            } else {
            	if (document.getSuffixName().equalsIgnoreCase("pdf")) {
            		try {
						List<com.yizhi.comment.application.vo.domain.PdfPagesVo> list = pdfPagesClient.list(id, TpActivityType.TYPE_DOCUMENT);
						List<PdfPagesVo> pagesVos=new ArrayList<>();

						if (!CollectionUtils.isEmpty(list)) {
                            for (com.yizhi.comment.application.vo.domain.PdfPagesVo p:list) {
                                PdfPagesVo pdfPagesVo=new PdfPagesVo();
                                pdfPagesVo.setId(p.getId());
                                pdfPagesVo.setBizId(p.getBizId());
                                pdfPagesVo.setBizType(p.getBizType());
                                pdfPagesVo.setImageUrl(p.getImageUrl());
                                pdfPagesVo.setPdfPage(p.getPdfPage());
                                pagesVos.add(pdfPagesVo);
                            }
							vo.setPdfPageVos(pagesVos);
						}
					} catch (Exception e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
            	vo.setSuffixName(document.getSuffixName());
				vo.setUrl(document.getUrl());
				vo.setIsVideo(document.getIsVideo());
				vo.setVideoSd(document.getVideoSd());
				vo.setVideoHd(document.getVideoHd());
				vo.setVideoLd(document.getVideoLd());
                return vo;
            }
        }
        return null;
    }

    @GetMapping("/download")
    public ParamsVo download(@RequestParam("id") Long id, @RequestParam("termialType") Integer termialType) {

        return documentService.download(id, termialType);
    }

    @GetMapping("/query/ByRelationId")
    public Page<DocumentVo> queryByIds(@RequestParam("relationId") Long relationId,
                                       @RequestParam("relationType") Integer relationType,
                                       @RequestParam(value = "pageNo", defaultValue = "1") Integer pageNo,
                                       @RequestParam(value = "pageSize", defaultValue = "10") Integer pageSize
    ) {

        return documentService.queryByRelationId(relationId, relationType, pageNo, pageSize);

    }

    //导出资料列表
    @PostMapping("/export/list")
    public void export(@RequestBody DocumentParamVo documentParamVo) {
        RequestContext context = ContextHolder.get();
        documentParamVo.setContext(context);
        System.out.println("===================进入下载=======================！");
        documentListExcel.execute(documentParamVo, true);
    }

    @GetMapping("/initalData")
    public void test() {

        List<Document> list = documentMapper.initalData();
        if (!CollectionUtils.isEmpty(list)) {
            for (Document a : list) {

                String s = workUtil.polyUploadVid(a.getFileName(), a.getUrl());
                LOGGER.info("返回的vid:" + s);
                if (s == null) {
                    continue;
                }
                Document document = new Document();
                document.setId(a.getId());
                document.setVId(s);
                documentService.updateById(document);
            }
        }
    }
    
    @PostMapping("/verifyVedio")
    public boolean verifyVedio(@RequestBody List<Long> documentIds) {
        int num = documentService.verifyVideo(documentIds);
        return num == documentIds.size();
    }
    
    @PostMapping("updateVedioTranscodeUrl")
    public boolean updateVedioTranscodeUrl(@RequestBody Map<String, String> map) {
        Long id = Long.valueOf(map.get("id"));
        String videoLd = map.get("ld");
        String videoHd = map.get("hd");
        String videoSd = map.get("sd");
        return documentService.updateVedioTranscodeUrl(id, videoLd, videoHd, videoSd);
    }
}

