package com.yizhi.documents.application.download;

import com.baomidou.mybatisplus.plugins.Page;
import com.yizhi.documents.application.service.ClassifyService;
import com.yizhi.documents.application.service.DocumentService;
import com.yizhi.documents.application.vo.documents.DocumentParamVo;
import com.yizhi.documents.application.vo.documents.DocumentVo;
import com.itextpdf.text.log.Logger;
import com.itextpdf.text.log.LoggerFactory;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.context.TaskContext;
import com.yizhi.core.application.file.constant.FileConstant;
import com.yizhi.core.application.file.task.AbstractDefaultTask;
import com.yizhi.core.application.file.util.OssUpload;
import org.apache.poi.hssf.usermodel.HSSFCellStyle;
import org.apache.poi.hssf.util.HSSFColor;
import org.apache.poi.ss.usermodel.HorizontalAlignment;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.xssf.usermodel.*;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;
import org.springframework.util.CollectionUtils;

import javax.annotation.Resource;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
@Component
public class DocumentListExcel extends AbstractDefaultTask<String, DocumentParamVo> {

    @Resource
    private DocumentService documentService;

    @Resource
    private ClassifyService classifyService;
    private static final Logger logger =  LoggerFactory.getLogger(DocumentListExcel.class);

    private SimpleDateFormat dateFormat = new SimpleDateFormat("yyyyMMddHHmm");
    private SimpleDateFormat dateFormat1 = new SimpleDateFormat("yyyy/MM/dd HH:mm");

    @Override
    protected String execute(DocumentParamVo paramVo) {
        System.out.println("开始创建下载任务！");
        RequestContext context = paramVo.getContext();
        Long companyId = context.getCompanyId();
        Long siteId = context.getSiteId();
        Long accountId = context.getAccountId();


        String taskName = paramVo.getTaskName();
        String serialNo = paramVo.getSerialNo();
        Long taskId = paramVo.getTaskId();
        paramVo.setEnableFilter(false);//不过滤
        paramVo.setPageNo(1);
        paramVo.setPageSize(Integer.MAX_VALUE);
        Date taskDate = new Date();

        TaskContext taskContext = new TaskContext(taskId, serialNo, taskName, accountId, taskDate, siteId, companyId);
        System.out.println("开始创建下载任务 第二步！");
        working(taskContext);
        System.out.println("开始创建下载任务 第三步！");



        //查询出需要的数据
        Integer dataNum = 0;
        List<DocumentVo> records = null;
        Page<DocumentVo> page = documentService.queryOnCondition(paramVo);
        System.out.println("开始创建下载任务 第四步！");
        Map<Long, String> classifyNameMap = new HashMap<>();

        if (page != null && page.getRecords() != null) {
            dataNum = page.getTotal();
            records = page.getRecords();
            //获取所有的分类id
            classifyNameMap = classifyService.getClassifyName(paramVo.getContext());
        }

        System.out.println("开始输出文件！");

        //需要返回的URL
        String upLoadUrl = null;
        try {
            //开始创建excel
            XSSFWorkbook workbook = new XSSFWorkbook();
            String sheetName = "资料信息列表" + "_" + dateFormat.format(taskDate);
            XSSFSheet sheet = workbook.createSheet(sheetName);
            // 创建第1行
            XSSFRow row = sheet.createRow((int) 0);
            //创建单元格格式
            XSSFCellStyle style = workbook.createCellStyle();
            style.setAlignment(HorizontalAlignment.CENTER);//创建居中格式
            style.setFillForegroundColor(HSSFColor.GREY_25_PERCENT.index);
           // style.setFillPattern(HSSFCellStyle.SOLID_FOREGROUND);
            XSSFCell cell = null;

            //创建标题行
            row = fillTitle(style, cell, row);
            System.out.println("开始填充数据！");
            //开始填充数据
            fillData(sheet, row, records, classifyNameMap);


            //上传到OSS,返回url
            upLoadUrl = outputFile(upLoadUrl, sheetName, workbook, taskContext);

            System.out.println("导出成功！"+"文件路径为"+upLoadUrl);
        } catch (Exception e) {
            e.printStackTrace();
            fail(taskContext, "资料库导出过程中发生错误，请查看日志");
            logger.error("创建Excel的过程中发生错误");
        }
        return upLoadUrl;
    }


    private XSSFRow fillTitle(XSSFCellStyle style, XSSFCell cell, XSSFRow row) {

        cell = row.createCell((short) 0);
        cell.setCellStyle(style);
        cell.setCellValue("资料编码");
        cell = row.createCell((short) 1);
        cell.setCellStyle(style);
        cell.setCellValue("资料名称");
        cell = row.createCell((short) 2);
        cell.setCellStyle(style);
        cell.setCellValue("资料类型");
        cell = row.createCell((short) 3);
        cell.setCellStyle(style);
        cell.setCellValue("上传者");
        cell = row.createCell((short) 4);
        cell.setCellStyle(style);
        cell.setCellValue("上传/更新时间");
        cell = row.createCell((short) 5);
        cell.setCellStyle(style);
        cell.setCellValue("资料大小");
        cell = row.createCell((short) 6);
        cell.setCellStyle(style);
        cell.setCellValue("下载次数");
        cell = row.createCell((short) 7);
        cell.setCellStyle(style);
        cell.setCellValue("资料分类");
        return row;
    }

    private void fillData(XSSFSheet sheet, XSSFRow row, List<DocumentVo> records, Map<Long, String> classifyNameMap) {
        if (!CollectionUtils.isEmpty(records)) {


            Integer rowIndex = 1;
            int i = 1;
            int y = 0;
            for (DocumentVo vo : records) {
                row = sheet.createRow(i++);
                //开始给每个单元格赋值
                DocumentVo documentVo = records.get(y++);
                Integer cellSize = 0;
                row.createCell(cellSize).setCellValue(documentVo.getCodeName());
                cellSize++;
                row.createCell(cellSize).setCellValue(documentVo.getName());
                cellSize++;
                row.createCell(cellSize).setCellValue(documentVo.getTypeName());
                cellSize++;
                row.createCell(cellSize).setCellValue(documentVo.getCreateAccounName());
                cellSize++;
                String time = dateFormat1.format(documentVo.getUpdateTime());
                row.createCell(cellSize).setCellValue(time);
                cellSize++;
                row.createCell(cellSize).setCellValue(documentVo.getFileSizeName());
                cellSize++;
                row.createCell(cellSize).setCellValue(documentVo.getDownloadTimes());
                cellSize++;
                String name = null;
                if (classifyNameMap != null) {
                    name = classifyNameMap.get(documentVo.getClassifyId());
                }
                row.createCell(cellSize).setCellValue(name);
            }
        }
    }

    private String outputFile(String upLoadUrl, String sheetName, Workbook workbook, TaskContext taskContext) throws IOException {
        String requestPath = FileConstant.SAVE_PATH;
//        String requestPath = "E\\";
        File fileDir = new File(requestPath);
        if (!fileDir.exists()) {
            fileDir.mkdir();
        }

        StringBuffer fileNameSb = new StringBuffer().append(sheetName).append(".xlsx");
        String fileName = fileNameSb.toString();
        String path = new StringBuffer().append(requestPath).append(fileNameSb).toString();
        FileOutputStream os = null;
        File file = null;
        try {
            os = new FileOutputStream(path);
            workbook.write(os);
            //阿里云返回url
            upLoadUrl = OssUpload.upload(path, fileName);
            file = new File(path);
            success(taskContext, "成功", upLoadUrl);
            System.out.println("往任务里面放入文件路径"+taskContext.getTaskName());
        } catch (Exception e1) {
            e1.printStackTrace();
            fail(taskContext, "写入过程中发生错误");
            logger.error("写入数据到Excel的过程中或者上传到阿里云中发生错误");
        } finally {
            if (os != null) {
                os.close();
            }
            if (workbook != null) {
                workbook.cloneSheet(0);
            }
            if (file != null) {
                file.delete();
            }
        }

        return upLoadUrl;
    }


}

