package com.yizhi.documents.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.documents.application.domain.DocumentClassify;
import com.yizhi.documents.application.mapper.ClassifyMapper;
import com.yizhi.documents.application.service.ClassifyService;
import com.yizhi.documents.application.util.WorkUtil;
import com.yizhi.documents.application.vo.documents.DocumentClassifyTreeVo;
import com.yizhi.documents.application.vo.documents.DocumentClassifyVo;
import com.yizhi.documents.application.vo.documents.SecondClassifyVo;
import com.yizhi.documents.application.vo.documents.ThirdClassifyVo;
import com.yizhi.application.orm.hierarchicalauthorization.HQueryUtil;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.Statement;
import java.util.*;


/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author hutao
 * @since 2019-06-11
 */
@Service
@Transactional
public class ClassifyServiceImple extends ServiceImpl<ClassifyMapper, DocumentClassify> implements ClassifyService {
    @Autowired
    IdGenerator idGenerator;
    @Autowired
    WorkUtil workUtil;

    private static final Logger LOGGER = LoggerFactory.getLogger(ClassifyServiceImple.class);

    @Override
    public Boolean save(DocumentClassify classify) {
        RequestContext context = ContextHolder.get();
        Long id = classify.getId();
        Long parentId = classify.getParentId();
        Long rootId = classify.getRootId();
        Date curentDate = new Date();

        if (null == id) {
            //id为null，进行插入操作
            classify.setId(idGenerator.generate());
            classify.setCompanyId(context.getCompanyId());
            classify.setSiteId(context.getSiteId());
            classify.setOrgId(context.getOrgId());
            classify.setCreateByName(context.getAccountName());
            classify.setCreateById(context.getAccountId());
            classify.setCreateTime(curentDate);
            classify.setDeleted(0);

            //parentId、rootId都为0，一级分类
            //parentId不为0，rootId为0，二级分类
            //parentId、rootId都不为0，三级分类
            if (parentId != 0) {
                if (rootId != 0) {
                    //该新建分类为三级分类
                    classify.setLayer(3);
                } else {
                    //该新建分类为二级分类
                    classify.setLayer(2);
                    classify.setRootId(parentId);
                }
                this.baseMapper.insert(classify);
            } else {
                //该新建分类为一级分类
                classify.setParentId(0l);
                classify.setRootId(0l);
                classify.setLayer(1);
                //完整的分类编码=为“code--name” code 只存储数字部分

                String key = context.getSiteId().toString();
                Boolean hasLock = false;
                //之所以这么写是因为 不知道为什么在查询code最大值时，会随机性拼接createById
                StringBuilder sql = new StringBuilder(" select Max(code) as code from classify c where c.layer =1 and c.deleted = 0 and c.site_id = ");
                sql.append(context.getSiteId()).append(" and c.company_id = ").append(context.getCompanyId());
                Connection connection = null;
                try {
                    connection = this.sqlSessionBatch().getConnection();
                    Statement statement = connection.createStatement();
                    LOGGER.info("执行的sql={}", sql.toString());
                    ResultSet rs = statement.executeQuery(sql.toString());
                    while (rs.next()){
                        Integer code = rs.getInt("code");
                        code = (code == null ? 999 : code) + 1;
                            classify.setCode(code);
                            this.baseMapper.insert(classify);
                    }
                    rs.close();
                } catch (Exception e) {
                    e.printStackTrace();
                }
//                while (!hasLock) {
//                    //谁锁住了谁执行 true 为当前线程锁住 false说明已被其他人锁住啦
//                    if (workUtil.lock(key)) {
//                        try {
//                            Integer code = this.baseMapper.queryMaxCode(context.getCompanyId(), context.getSiteId(), context.getOrgId());
//                            code = (code == null ? 999 : code) + 1;
//                            classify.setCode(code);
//                            this.baseMapper.insert(classify);
//                        } catch (Exception e) {
//                            e.printStackTrace();
//                        } finally {
//                            hasLock = true;
//                            //释放锁
//                            workUtil.dealLock(key);
//                        }
//                    }
//                }
            }
            return true;
        }
        return false;
    }


    @Override
    public Boolean update(DocumentClassifyVo classifyVo) {
        RequestContext context = ContextHolder.get();
        Date curentDate = new Date();

        if (null != classifyVo) {
            DocumentClassify updateClassify = new DocumentClassify();

            if (classifyVo.getParentId() != null && classifyVo.getParentId() != 0) {
                DocumentClassify parentClassify = this.baseMapper.selectById(classifyVo.getParentId());
                if (parentClassify != null) {
                    //获取层级
                    Integer layer = parentClassify.getLayer();

                    if (layer == 1) {
                        updateClassify.setRootId(parentClassify.getId());
                    } else {
                        updateClassify.setRootId(parentClassify.getParentId());
                    }
                    updateClassify.setLayer((++layer));
                    updateClassify.setParentId(classifyVo.getParentId());
                } else {
                    return false;
                }
            }
            updateClassify.setId(classifyVo.getId());
            updateClassify.setName(classifyVo.getName());
            updateClassify.setDescription(classifyVo.getDescription());
            updateClassify.setUpdateTime(curentDate);
            updateClassify.setUpdateByName(context.getAccountName());
            updateClassify.setUpdateById(context.getAccountId());
            try {
                if (this.baseMapper.updateById(updateClassify) == 1) {
                    return true;
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return false;
    }

    @Override
    public Boolean delete(DocumentClassify classify) {
        classify.setDeleted(1);
        try {
            this.baseMapper.updateById(classify);
            return true;
        } catch (Exception e) {
            e.printStackTrace();
        }
        return false;
    }

    @Override
    public List<DocumentClassifyTreeVo> queryClassifyList() {
        RequestContext context = ContextHolder.get();
        //开启分级授权
        HQueryUtil.startHQ(DocumentClassify.class);
        List<DocumentClassifyTreeVo> classifyTreeVos = new ArrayList<>();
        List<DocumentClassifyVo> classifyVos = this.baseMapper.queryClassifyTree(context.getCompanyId(), context.getSiteId(), context.getOrgId());
        if (!CollectionUtils.isEmpty(classifyVos)) {
            //储存第一级分类  <id ,对应的VO>
            Map<Long, DocumentClassifyTreeVo> rootClassifyMap = new HashMap<>();
            //储存第二级分类 《id，对应的vo》
            Map<Long, SecondClassifyVo> secondClassifyMap = new HashMap<>();
            for (DocumentClassifyVo vo : classifyVos) {
                if (vo.getLayer() == 1) {
                    vo.setCodeName(vo.getCode() + "--" + vo.getName());
                    DocumentClassifyTreeVo classifyTreeVo = new DocumentClassifyTreeVo();
                    classifyTreeVo.setClassifyVo(vo);
                    classifyTreeVos.add(classifyTreeVo);
                    if (!rootClassifyMap.containsKey(vo.getId())) {
                        rootClassifyMap.put(vo.getId(), classifyTreeVo);
                    }
                }
            }
            for (DocumentClassifyVo vo : classifyVos) {
                if (vo.getLayer() == 2) {
                    if (null != rootClassifyMap.get(vo.getParentId())) {
                        DocumentClassifyTreeVo classifyTreeVo = rootClassifyMap.get(vo.getParentId());
                        if (null != classifyTreeVo) {
                            SecondClassifyVo secondClassifyVo = new SecondClassifyVo();
                            vo.setCodeName(vo.getName());
                            secondClassifyVo.setClassifyVo(vo);
                            classifyTreeVo.getChildren().add(secondClassifyVo);
                            if (!secondClassifyMap.containsKey(vo.getId())) {
                                secondClassifyMap.put(vo.getId(), secondClassifyVo);
                            }
                        }
                    }
                }

            }
            for (DocumentClassifyVo vo : classifyVos) {
                if (vo.getLayer() == 3) {
                    if (null != secondClassifyMap.get(vo.getParentId())) {
                        SecondClassifyVo secondClassifyVo = secondClassifyMap.get(vo.getParentId());
                        ThirdClassifyVo thirdClassifyVo = new ThirdClassifyVo();
                        vo.setCodeName(vo.getName());
                        thirdClassifyVo.setClassifyVo(vo);
                        if (null != secondClassifyVo) {
                            secondClassifyVo.getChildren().add(thirdClassifyVo);
                        }
                    }
                }

            }
        }
        return classifyTreeVos;
    }


    @Override
    public Boolean haveChildrenClassify(Long classifyId) {
        RequestContext context = ContextHolder.get();
        Integer count = this.baseMapper.haveChildrenClassify(classifyId, context.getCompanyId(), context.getSiteId(), context.getOrgId());
        if (count > 0) {
            return true;
        } else
            return false;

    }

    @Override
    public Map<Long, String> getClassifyName(RequestContext context) {
        Map<Long, String> classifyNameMap = null;
        DocumentClassify classifyTem = new DocumentClassify();
        classifyTem.setSiteId(context.getSiteId());
        classifyTem.setCompanyId(context.getCompanyId());

        List<DocumentClassify> classifiesAll = this.baseMapper.selectList(new EntityWrapper<>(classifyTem));
        if (!CollectionUtils.isEmpty(classifiesAll)) {
            classifyNameMap = new HashMap<>(classifiesAll.size());
            for (DocumentClassify classify : classifiesAll) {
                if (!classifyNameMap.containsKey(classify.getId())) {
                    classifyNameMap.put(classify.getId(), classify.getName());
                }
            }
            for (DocumentClassify classify : classifiesAll) {

                if (classify.getLayer() == 2) {
                    String name = classifyNameMap.get(classify.getId());
                    String parentName = classifyNameMap.get(classify.getParentId());
                    classifyNameMap.put(classify.getId(), (parentName + "/" + name));
                } else if (classify.getLayer() == 3) {
                    String name = classifyNameMap.get(classify.getId());
                    String parentName = classifyNameMap.get(classify.getParentId());
                    classifyNameMap.put(classify.getId(), parentName + "/" + name);
                }
            }


        }
        return classifyNameMap;
    }


}
