package com.yizhi.documents.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.documents.application.domain.DocumentRelation;
import com.yizhi.documents.application.mapper.DocumentRelationMapper;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.documents.application.service.DocumentRelationService;
import com.yizhi.documents.application.vo.documents.DocumentRelationVo;
import com.yizhi.documents.application.vo.documents.ParamsVo;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

import java.util.Date;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author hutao
 * @since 2019-06-11
 */
@Service
public class DocumentRelationServiceImple extends ServiceImpl<DocumentRelationMapper, DocumentRelation> implements DocumentRelationService {

    @Autowired
    private IdGenerator idGenerator;

    private final static Integer MOVE_UP = 1;
    private final static Integer MOVE_DOWN = 2;




    @Override
    public Boolean delete(Long id, Long documentId) {
        RequestContext context = ContextHolder.get();
        Integer i = this.baseMapper.delete(id, documentId, context.getCompanyId(), context.getSiteId(), context.getOrgId());
        if (i > 0 ) {
            return true;
        }
        return false;
    }

    @Override
    public Boolean hasRelation(Long documentId) {
        DocumentRelation documentRelation = new DocumentRelation();
        documentRelation.setDocumentId(documentId);
        documentRelation.setDeleted(0);
        EntityWrapper wrapper = new EntityWrapper(documentRelation);
        if (this.baseMapper.selectCount(wrapper) > 0) {
            return true;
        }
        return false;
    }

    @Override
    public List<Long> returnDocumentIds(Long relationId, Integer relationType) {
        RequestContext context = ContextHolder.get();

        return this.baseMapper.returnDocumentIds(relationId, relationType, context.getCompanyId(), context.getSiteId(), context.getOrgId());
    }

    @Override
    public Boolean batchSave(List<Long> documentIds, Long relationId, Integer relationType) {
        if (CollectionUtils.isEmpty(documentIds)) {
            return false;
        }

        Date curentDate = new Date();
        RequestContext context = ContextHolder.get();

        for (Long documentId : documentIds) {
            DocumentRelation relation = new DocumentRelation();
            relation.setId(idGenerator.generate());
            relation.setRelationId(relationId);
            relation.setRelationType(relationType);
            relation.setDocumentId(documentId);

            Integer maxSort = this.baseMapper.queryMaxSort(relationId, relationType, context.getCompanyId(), context.getSiteId(), context.getOrgId());
            maxSort = maxSort == null ? 1 : maxSort;
            relation.setSort((++maxSort));

            relation.setCompanyId(context.getCompanyId());
            relation.setSiteId(context.getSiteId());
            relation.setOrgId(context.getOrgId());
            relation.setCreateByName(context.getAccountName());
            relation.setCreateById(context.getAccountId());
            relation.setCreateTime(curentDate);
            relation.setDeleted(0);
            try {
                this.baseMapper.insert(relation);
            } catch (Exception e) {
                e.printStackTrace();
                return false;
            }

        }

        return true;
    }

    @Override
    public ParamsVo move(DocumentRelationVo documentRelationVo) {

        RequestContext context = ContextHolder.get();
        ParamsVo paramsVo = new ParamsVo();

        DocumentRelation orginRelation = this.baseMapper.selectById(documentRelationVo.getId());

        if (orginRelation == null) {
            paramsVo.setErrorMsg("数据库不存在该id!");
            paramsVo.setSuccess(false);
            return paramsVo;
        } else {
            Integer orginSort = orginRelation.getSort();

            DocumentRelation targetRelation;
            //是否上移
            if (documentRelationVo.getMoveUp()) {
                targetRelation = this.baseMapper.queryBySort(orginSort, documentRelationVo.getRelationId(), documentRelationVo.getRelationType(), MOVE_UP, context.getCompanyId(), context.getSiteId(), context.getOrgId());
            } else {
                targetRelation = this.baseMapper.queryBySort(orginSort, documentRelationVo.getRelationId(), documentRelationVo.getRelationType(), MOVE_DOWN, context.getCompanyId(), context.getSiteId(), context.getOrgId());
            }
            if (targetRelation != null) {
                Integer targetSort = targetRelation.getSort();
                orginRelation.setSort(targetSort);
                targetRelation.setSort(orginSort);
                try {
                    Integer tem = this.baseMapper.updateById(orginRelation);
                    Integer tem1 = this.baseMapper.updateById(targetRelation);
                    if (tem == 1 && tem1 == 1) {
                        paramsVo.setSuccess(true);
                        paramsVo.setErrorMsg(null);
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            } else {
                paramsVo.setErrorMsg("移动失败！");
                paramsVo.setSuccess(false);
            }
        }
        return paramsVo;
    }
}
