package com.yizhi.documents.application.service.impl;

import com.baomidou.mybatisplus.mapper.EntityWrapper;
import com.baomidou.mybatisplus.plugins.Page;
import com.baomidou.mybatisplus.service.impl.ServiceImpl;
import com.yizhi.documents.application.domain.Document;
import com.yizhi.documents.application.domain.DownloadRecord;
import com.yizhi.documents.application.enums.DocumentType;
import com.yizhi.documents.application.enums.TermialType;
import com.yizhi.documents.application.mapper.ClassifyMapper;
import com.yizhi.documents.application.mapper.DocumentMapper;
import com.yizhi.documents.application.mapper.DocumentRelationMapper;
import com.yizhi.documents.application.service.DocumentRelationService;
import com.yizhi.documents.application.service.DocumentService;
import com.yizhi.documents.application.service.DownloadRecordService;
import com.yizhi.documents.application.util.CommonUtils;
import com.yizhi.documents.application.util.WorkUtil;

import com.yizhi.application.orm.hierarchicalauthorization.HQueryUtil;
import com.yizhi.application.orm.id.IdGenerator;
import com.yizhi.comment.application.feign.PdfPagesClient;
import com.yizhi.comment.application.vo.PdfVO;
import com.yizhi.comment.application.vo.domain.PdfPagesVo;
import com.yizhi.core.application.context.ContextHolder;
import com.yizhi.core.application.context.RequestContext;
import com.yizhi.core.application.event.EventWrapper;
import com.yizhi.core.application.publish.CloudEventPublisher;
import com.yizhi.documents.application.vo.documents.*;
import com.yizhi.util.application.constant.TpActivityType;
import com.yizhi.util.application.event.AssignmentMediaTranscodingEvent;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.CollectionUtils;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 服务实现类
 * </p>
 *
 * @author hutao
 * @since 2019-06-11
 */
@Service
@Transactional
public class DocumentServiceImple extends ServiceImpl<DocumentMapper, Document> implements DocumentService {

    @Autowired
    private IdGenerator idGenerator;
    @Autowired
    private WorkUtil workUtil;
    @Autowired
    private DocumentMapper documentMapper;

    @Autowired
    private ClassifyMapper classifyMapper;

    @Autowired
    private DownloadRecordService downloadRecordService;

    @Autowired
    private DocumentRelationMapper relationMapper;

    @Autowired
    private DocumentRelationService documentRelationService;

    @Autowired
    private CommonUtils commonUtils;
    
    private SimpleDateFormat sdf = new SimpleDateFormat("yyyyMMdd");
    
    @Autowired
	private CloudEventPublisher cloudEventPublisher;

    @Autowired
    private PdfPagesClient pdfPagesClient;
    
    @Override
    public Integer queryRelationCount(Long classfyId) {

        Document document = new Document();
        document.setClassifyId(classfyId);
        document.setDeleted(0);
        EntityWrapper entityWrapper = new EntityWrapper(document);
        return this.baseMapper.selectCount(entityWrapper);
    }

    @Override
    public Boolean upload(UploadVo uploadVo) {
        RequestContext context = ContextHolder.get();
        Date curentDate = new Date();
        if (null != uploadVo) {
            Document document = new Document();
            document.setId(idGenerator.generate());
            document.setClassifyId(uploadVo.getClassfyId());
            document.setFileName(uploadVo.getFileName());
            document.setFileSize(uploadVo.getFileSize());
            document.setType(workUtil.documentType(uploadVo.getSuffixName()));
            document.setUId(uploadVo.getUuId());
            document.setDownloadTimes(0);
            document.setName(uploadVo.getName());
            document.setUrl(uploadVo.getUrl());
            String dateFormat = sdf.format(curentDate);
            Integer code = documentMapper.queryMaxCode(dateFormat, context.getCompanyId(), context.getSiteId(), context.getOrgId());
            code = (code == null ? 1 : ++code);
            document.setCode(code);
            //给编码名称赋值
            document.setCodeName(getCodeName(code, dateFormat));
            document.setCompanyId(context.getCompanyId());
            document.setSiteId(context.getSiteId());
            document.setOrgId(context.getOrgId());
            document.setCreateByName(context.getAccountName());
            document.setCreateById(context.getAccountId());
            document.setCreateTime(curentDate);
            document.setDeleted(0);
            document.setSuffixName(uploadVo.getSuffixName());
            document.setKeywords(uploadVo.getKeywords());
            if (workUtil.documentType(uploadVo.getSuffixName())== DocumentType.VIDEO.getType()||workUtil.documentType(uploadVo.getSuffixName())==DocumentType.AUDIO.getType()) {
				AssignmentMediaTranscodingEvent event = new AssignmentMediaTranscodingEvent();
				List<AssignmentMediaTranscodingEvent.FileDTO> fileDTOS = new ArrayList<>();
				fileDTOS.add(event.new FileDTO(document.getId(), document.getUrl()));
				event.setType(TpActivityType.TYPE_DOCUMENT);
				event.setFiles(fileDTOS);
            	cloudEventPublisher.publish("assignmentMediaTranscoding",
						new EventWrapper(0L, event));
			}
            if (uploadVo.getSuffixName().equalsIgnoreCase("pdf")) {
				List<PdfVO> pdfVOs = new ArrayList<PdfVO>();
                PdfVO pdfVO = new PdfVO();
                pdfVO.setBizId(document.getId());
                pdfVO.setBizType(TpActivityType.TYPE_DOCUMENT);
                pdfVO.setCompanyId(document.getCompanyId());
                pdfVO.setSiteId(document.getSiteId());
                pdfVO.setCreateById(document.getCreateById());
                pdfVO.setOriginalFile(document.getFileName());
                pdfVO.setOriginalFileUrl(document.getUrl());
                pdfVOs.add(pdfVO);
				try {
					if (!CollectionUtils.isEmpty(pdfVOs)) {
						cloudEventPublisher.publish("pdfToPng",
								new EventWrapper(0L, pdfVOs));
					}
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
            try {
                Integer insert = documentMapper.insert(document);
                return true;
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        return false;
    }

    private String getCodeName(Integer code, String dateFormat) {
        String codeName = null;
        if (code > 0 && code < 10) {
            codeName = dateFormat + "00" + code;
        } else if (code >= 10 && code < 100) {
            codeName = dateFormat + "0" + code;
        } else {
            codeName = dateFormat + code;
        }
        return codeName;
    }

    @Override
    public Page<DocumentVo> queryOnCondition(DocumentParamVo paramVo) {
        RequestContext context = paramVo.getContext();
        ContextHolder.set(context);
        System.out.println("开始查询资料！");
        //开启分级授权
        HQueryUtil.startHQ(Document.class);

        Long conpanyId = context.getCompanyId();
        Long siteId = context.getSiteId();
        Long orgId = context.getOrgId();
        String startDate = paramVo.getStartDate();
        String endDate = paramVo.getEndDate();
        endDate = endDate+" 23:59:59";
        Long classifyId = paramVo.getClassifyId();
        List<Long> classifyIds = null;
        if (classifyId != null) {
            classifyIds = classifyMapper.queryChildrenClassify(classifyId, conpanyId, siteId, orgId);
        }
        String name = paramVo.getName();
        Integer type = paramVo.getType();
        Page page = new Page(paramVo.getPageNo(), paramVo.getPageSize());
        List<DocumentVo> documentVos = new ArrayList<>();
        //需要过滤的资料ids
        List<Long> filterList = null;
        //查询已关联的资料，进行过滤

        if (paramVo.getEnableFilter()) {
            if (paramVo.getRelationId() != null && paramVo.getRelationType() != null) {
                filterList = relationMapper.getFilterDocumentIds(paramVo.getRelationId(), paramVo.getRelationType(), conpanyId, siteId, orgId);
            }
        }

        documentVos = documentMapper.queryOnCondition(classifyIds, name, type, filterList, conpanyId, siteId, orgId, paramVo.getEnableTypeFilter(), page,startDate,endDate);
        if (!CollectionUtils.isEmpty(documentVos)) {
            Integer no = 0;
            for (DocumentVo a : documentVos) {
                ++no;
                //组装数据
                a = fillDate(a, no);
            }
        }
        page.setRecords(documentVos);
        return page;
    }


    @Override
    public ParamsVo update(UploadVo uploadVo) {

        Date curentDate = new Date();
        ParamsVo paramsVo = new ParamsVo();
        RequestContext context = ContextHolder.get();
        Document document = documentMapper.selectById(uploadVo.getId());
        if (null == document) {
            paramsVo.setErrorMsg("id不存在！");
            paramsVo.setSuccess(false);
        } else {
            document.setUrl(uploadVo.getUrl());
            document.setName(uploadVo.getName());
            document.setFileName(uploadVo.getFileName());
            document.setFileSize(uploadVo.getFileSize());
            document.setUpdateById(context.getAccountId());
            document.setUpdateByName(context.getAccountName());
            document.setUpdateTime(curentDate);
            document.setType(workUtil.documentType(uploadVo.getSuffixName()));
            document.setSuffixName(uploadVo.getSuffixName());
            document.setKeywords(uploadVo.getKeywords());
            if (workUtil.documentType(uploadVo.getSuffixName())==DocumentType.VIDEO.getType()||workUtil.documentType(uploadVo.getSuffixName())==DocumentType.AUDIO.getType()) {
				AssignmentMediaTranscodingEvent event = new AssignmentMediaTranscodingEvent();
				List<AssignmentMediaTranscodingEvent.FileDTO> fileDTOS = new ArrayList<>();
				fileDTOS.add(event.new FileDTO(document.getId(), document.getUrl()));
				event.setType(TpActivityType.TYPE_DOCUMENT);
				event.setFiles(fileDTOS);
            	cloudEventPublisher.publish("assignmentMediaTranscoding",
						new EventWrapper(0L, event));
			}
            if (uploadVo.getSuffixName().equalsIgnoreCase("pdf")) {
				List<PdfVO> pdfVOs = new ArrayList<PdfVO>();
                PdfVO pdfVO = new PdfVO();
                pdfVO.setBizId(document.getId());
                pdfVO.setBizType(TpActivityType.TYPE_DOCUMENT);
                pdfVO.setCompanyId(document.getCompanyId());
                pdfVO.setSiteId(document.getSiteId());
                pdfVO.setCreateById(document.getUpdateById());
                pdfVO.setOriginalFile(document.getFileName());
                pdfVO.setOriginalFileUrl(document.getUrl());
                pdfVOs.add(pdfVO);
				try {
					if (!CollectionUtils.isEmpty(pdfVOs)) {
						cloudEventPublisher.publish("pdfToPng",
								new EventWrapper(0L, pdfVOs));
					}
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
            try {
                if (documentMapper.updateById(document) == 1) {
                    paramsVo.setSuccess(true);
                    paramsVo.setErrorMsg(null);
                }
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return paramsVo;
    }

    @Override
    public ParamsVo download(Long id, Integer termialType) {

        ParamsVo paramsVo = new ParamsVo();
        Document document = this.documentMapper.selectById(id);
        if (document == null) {
            paramsVo.setSuccess(false);
            paramsVo.setErrorMsg("id不存在！");
            return paramsVo;
        } else {

            //向下载记录表 插一条记录
            Boolean saveSuccess = null;
            try {
                DownloadRecord downloadRecord = new DownloadRecord();
                downloadRecord.setTermialType(termialType);
                downloadRecord.setDocumentId(id);
                downloadRecord.setDocumentName(document.getName());
                saveSuccess = downloadRecordService.save(downloadRecord);
            } catch (Exception e) {
                e.printStackTrace();
                throw e;
            }
            if (saveSuccess == false) {
                paramsVo.setSuccess(false);
                paramsVo.setErrorMsg("下载失败!");
                return paramsVo;
            }

            //管理端下载   不算下载次数
            if (!TermialType.MANAGE.getType().equals(termialType)) {

                document.setDownloadTimes((document.getDownloadTimes() + 1));
                DocumentVo vo = new DocumentVo();
                vo.setSuffixName(document.getSuffixName());
                vo.setUrl(document.getUrl());
				vo.setIsVideo(document.getIsVideo());
				vo.setVideoSd(document.getVideoSd());
				vo.setVideoHd(document.getVideoHd());
				vo.setVideoLd(document.getVideoLd());
            	if (document.getSuffixName().equalsIgnoreCase("pdf")) {
            		try {
						List<PdfPagesVo> list = pdfPagesClient.list(id, TpActivityType.TYPE_DOCUMENT);
                        List<com.yizhi.documents.application.vo.documents.PdfPagesVo> pagesVos=new ArrayList<>();
						if (!CollectionUtils.isEmpty(list)) {
                            for (PdfPagesVo p:list) {
                                com.yizhi.documents.application.vo.documents.PdfPagesVo pdfPagesVo=new com.yizhi.documents.application.vo.documents.PdfPagesVo();
                                pdfPagesVo.setId(p.getId());
                                pdfPagesVo.setBizId(p.getBizId());
                                pdfPagesVo.setBizType(p.getBizType());
                                pdfPagesVo.setImageUrl(p.getImageUrl());
                                pdfPagesVo.setPdfPage(p.getPdfPage());
                                pagesVos.add(pdfPagesVo);
                            }
                            vo.setPdfPageVos(pagesVos);
						}
					} catch (Exception e) {
						// TODO Auto-generated catch block
						e.printStackTrace();
					}
				}
                paramsVo.setSuccess(true);
                paramsVo.setErrorMsg(null);
                paramsVo.setSuccessData(vo);
                
            } else {
                paramsVo.setSuccess(true);
                paramsVo.setErrorMsg(null);
                paramsVo.setSuccessData(document.getUrl());
            }
        }
        return paramsVo;
    }

    @Override
    public Page<DocumentVo> queryByRelationId(Long relationId, Integer relationType, Integer pageNo, Integer pageSize) {
        //开启分级授权
        HQueryUtil.startHQ(Document.class);
        RequestContext context = ContextHolder.get();

        Page<DocumentVo> page = new Page<>(pageNo, pageSize);
        System.out.println(context.getCompanyId() + "=====" + context.getSiteId());
        List<DocumentVo> documentVos = this.baseMapper.queryByRelationId(relationId, relationType, context.getCompanyId(), context.getSiteId(), page);

        Integer no = (pageNo - 1) * pageSize;
        for (DocumentVo a : documentVos) {
            ++no;
            //组装数据
            a = fillDate(a, no);
        }

//        List<Long> documentIds = documentRelationService.returnDocumentIds(relationId, relationType);
//        page.setTotal(documentIds == null ? 0 : documentIds.size());
        page.setRecords(documentVos);

        return page;
    }


    /**
     * @param a
     * @param no 序号
     * @return
     */
    private DocumentVo fillDate(DocumentVo a, Integer no) {

        String fileSizeName = null;
        //给资料大小名称附上单位
        if (a.getFileSize() >= (1024 * 1024)) {
            //只保留一位小数
            fileSizeName = String.format("%.1f", a.getFileSize() / (1024 * 1024)) + "M";

        } else {
            fileSizeName = String.format("%.1f", a.getFileSize() / 1024) + "K";
        }
        a.setFileSizeName(fileSizeName);
        //给序号赋值
        a.setNo((no));
        //将文件类型变成中文名称
        switch (a.getType()) {
            case 1:
                a.setTypeName(DocumentType.DOCUMENT.getTypeName());
                break;
            case 2:
                a.setTypeName(DocumentType.PICTURE.getTypeName());
                break;
            case 3:
                a.setTypeName(DocumentType.AUDIO.getTypeName());
                break;
            case 4:
                a.setTypeName(DocumentType.VIDEO.getTypeName());
                break;
            case 5:
                a.setTypeName(DocumentType.OTHER.getTypeName());
                break;
        }
        return a;
    }
    
    @Override
    public Integer verifyVideo(List<Long> ids) {
        return this.baseMapper.verifyVideo(ids);
    }

	@Override
	public Boolean updateVedioTranscodeUrl(Long id, String videoLd, String videoHd, String videoSd) {
		// TODO Auto-generated method stub
		return this.baseMapper.updateVedioTranscodeUrl(id, videoLd, videoHd, videoSd);
	}
}
