package com.yizhi.documents.application.util;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.yizhi.documents.application.enums.DocumentType;
import com.yizhi.core.application.cache.RedisCache;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.Map;

@Component
public class WorkUtil {
    @Autowired
    private RedisCache redisCache;

    private static final int expireTime = 3;
    @Value("${polyv.remote.url}")
    private String polyvUrl;

    @Value("${polyv.writetoken}")
    private String writeToken;
    
    @Value("${polyv.cataId}")
    private String cataId;

    private Logger LOGGER =  LoggerFactory.getLogger(WorkUtil.class);


    public  Integer documentType(String suffixName) {

        if (suffixName.equalsIgnoreCase("pdf") || suffixName.equalsIgnoreCase("doc") || suffixName.equalsIgnoreCase("xlsx")
                || suffixName.equalsIgnoreCase("xls") || suffixName.equalsIgnoreCase("ppt")
                || suffixName.equalsIgnoreCase("pptx") || suffixName.equalsIgnoreCase("docx")) {

            return DocumentType.DOCUMENT.getType();
        }

        if (suffixName.equalsIgnoreCase("bmp") || suffixName.equalsIgnoreCase("jpg") || suffixName.equalsIgnoreCase("png") || suffixName.equalsIgnoreCase("tif")
                || suffixName.equalsIgnoreCase("gif") || suffixName.equalsIgnoreCase("pcx") || suffixName.equalsIgnoreCase("tga")
                || suffixName.equalsIgnoreCase("exif") || suffixName.equalsIgnoreCase("fpx") || suffixName.equalsIgnoreCase("svg")
                || suffixName.equalsIgnoreCase("psd") || suffixName.equalsIgnoreCase("cdr") || suffixName.equalsIgnoreCase("pcd")
                || suffixName.equalsIgnoreCase("dxf") || suffixName.equalsIgnoreCase("ufo") || suffixName.equalsIgnoreCase("eps")
                || suffixName.equalsIgnoreCase("ai") || suffixName.equalsIgnoreCase("raw") || suffixName.equalsIgnoreCase("WMF")
                || suffixName.equalsIgnoreCase("webp")|| suffixName.equalsIgnoreCase("jpeg")) {

            return DocumentType.PICTURE.getType();
        }

        if (suffixName.equalsIgnoreCase("mp3") || suffixName.equalsIgnoreCase("wma") || suffixName.equalsIgnoreCase("wav")
                || suffixName.equalsIgnoreCase("mp3pro") || suffixName.equalsIgnoreCase("cd") || suffixName.equalsIgnoreCase("wave")
                || suffixName.equalsIgnoreCase("aiff") || suffixName.equalsIgnoreCase("mpeg") || suffixName.equalsIgnoreCase("mpeg-4")
                || suffixName.equalsIgnoreCase("midi") || suffixName.equalsIgnoreCase("RealAudio") || suffixName.equalsIgnoreCase("VQF")
                || suffixName.equalsIgnoreCase("OggVorbis") || suffixName.equalsIgnoreCase("AMR") || suffixName.equalsIgnoreCase("APE")
                || suffixName.equalsIgnoreCase("FLAC") || suffixName.equalsIgnoreCase("AAC") || suffixName.equalsIgnoreCase("ogg")
                || suffixName.equalsIgnoreCase("flac") || suffixName.equalsIgnoreCase("aac")) {
            return DocumentType.AUDIO.getType();
        }

        if (suffixName.equalsIgnoreCase("mp4") || suffixName.equalsIgnoreCase("avi") || suffixName.equalsIgnoreCase("mov") || suffixName.equalsIgnoreCase("flv")
                || suffixName.equalsIgnoreCase("mvb") || suffixName.equalsIgnoreCase("MPEG") || suffixName.equalsIgnoreCase("ASF")
                || suffixName.equalsIgnoreCase("3GP") || suffixName.equalsIgnoreCase("MKV") || suffixName.equalsIgnoreCase("F4V")
                || suffixName.equalsIgnoreCase("RMVB") || suffixName.equalsIgnoreCase("RM") || suffixName.equalsIgnoreCase("MOV")
                || suffixName.equalsIgnoreCase("QT") || suffixName.equalsIgnoreCase("NAVI") || suffixName.equalsIgnoreCase("DivX")
                || suffixName.equalsIgnoreCase("MPG") || suffixName.equalsIgnoreCase("DAT") || suffixName.equalsIgnoreCase("rm")
                || suffixName.equalsIgnoreCase("vob") || suffixName.equalsIgnoreCase("rmvb") || suffixName.equalsIgnoreCase("mkv")
                || suffixName.equalsIgnoreCase("f4v") || suffixName.equalsIgnoreCase("3gp") || suffixName.equalsIgnoreCase("wmv")
                || suffixName.equalsIgnoreCase("vob")) {
            return DocumentType.VIDEO.getType();
        }

        return DocumentType.OTHER.getType();

    }


    public boolean lock(String cachKey) {
        String key = "lock_documents:" + cachKey;
        String value = "xxxx";
        Boolean result = redisCache.setIfAbsent(key, value);// 0 已经存在 1原始不存在
        if (result) {// 如果不存在，存储完，给设置过期时间
            redisCache.expire(key, expireTime);
            return true;
        }
        // 如果已经存在，证明有别的服务给设置过
        return false;
    }

    /**
     * 锁释放
     *
     * @param cachKey
     */
    public void dealLock(String cachKey) {
        String key = "lock_documents:" + cachKey;
        redisCache.delete(key);
    }


    /**
     * 上传保利威视
     *
     * @param
     * @date 2019年3月4日
     */

    public String polyUploadVid(String fileName, String fileUrl) {


        HttpClientUtils httpClientUtils = new HttpClientUtils();


        // 远程上传保利威视 视频
        Map<String, String> polyvMap = new HashMap<String, String>(16);
        polyvMap.put("fileUrl", fileUrl);
        polyvMap.put("writetoken", writeToken);
        polyvMap.put("title", fileName);
        polyvMap.put("cataid",cataId);
        LOGGER.info("参数的polyvMap：" + polyvMap);
        String reStr = httpClientUtils.httpPost(polyvUrl, polyvMap);
        LOGGER.info("上传保利威视的远程视的结果：" + reStr);

        JSONObject polyResult = JSON.parseObject(reStr);
        LOGGER.info("保利威视返回的结果：" + reStr);
        if (null != reStr && polyResult.containsKey("data")) {
            String data = polyResult.getString("data");
            JSONArray jsonArray = JSON.parseArray(data);
            JSONObject dat = JSON.parseObject(jsonArray.get(0).toString());
            String vid = dat.getString("vid");
            LOGGER.info("保利威视的vid：" + vid);
            return vid;
        } else {
            LOGGER.info("视频上传异常!");
            return null;
        }

    }
}